# Copyright (C) 2016, 2017 Cumulus Networks, Inc. All rights reserved
#
# This software is subject to the Cumulus Networks End User License Agreement available
# at the following locations:
#
# Internet: https://cumulusnetworks.com/downloads/eula/latest/view/
# Cumulus Linux systems: /usr/share/cumulus/EULA.txt

from restapi.utils.execute import execute_command_no_resp


class DiscoveryError(RuntimeError):
    pass


class DiscoveryManager(object):

    @classmethod
    def find_interface(cls, hostname):
        neighbors = cls.fetch_neighbors()

        for interface, details in neighbors.items():
            if details['chassis']['name'] == hostname:
                return interface
        # This causes a problem where neutron asks for serverX port
        # but it doesn't exist. If in a multi-VM bringup, causes neutron
        # to go nuts so for now just don't do it. If user actually forgot
        # to enable LLDP on server port. Then, too bad. it never gets
        # provisioned. Better way to handle this problem will be investigated.
        # For now, always remember to configure LLDP correctly.
        # Suggest even have PTM running to ensure LLDP is configured correctly
        # raise DiscoveryError('Could not find host: {}'.format(hostname))

    @classmethod
    def find_neighbor_for_interface(cls, interface):
        neighbors = cls.fetch_neighbors()

        try:
            return neighbors[interface]['chassis']
        except KeyError:
            pass

    @classmethod
    def fetch_neighbors(cls):
        cmd = 'lldpcli show neighbors -f keyvalue'
        result, _ = execute_command_no_resp(cmd)
        parsed = parse_lldpd_output(result)
        return parsed['lldp']


def parse_lldpd_output(output):
    result = {}
    entries = output.strip().split('\n')

    for entry in entries:
        path, value = entry.strip().split('=', 1)
        path = path.split('.')
        components, final = path[:-1], path[-1]

        current = result
        for component in components:
            current = current.setdefault(component, {})
        current[final] = value

    return result
