# Copyright (C) 2016, 2017 Cumulus Networks, Inc. All rights reserved
#
# This software is subject to the Cumulus Networks End User License Agreement available
# at the following locations:
#
# Internet: https://cumulusnetworks.com/downloads/eula/latest/view/
# Cumulus Linux systems: /usr/share/cumulus/EULA.txt

from restapi.utils.logs import get_logger
from restapi.apps.ml2.l2 import L2Manager
from restapi.apps.ml2.l2 import HashManager
import falcon
import json


_LOG = get_logger()

_NETWORKS = {
}

_PHYSICAL_INTERFACES = {
}

_BRIDGES = {
}


HASH_M = HashManager()


class NetworkCmd(object):

    def on_put(self, req, resp, network_id):
        body = req.stream.read()
        if not body:
            # XXX fixme
            resp.status = falcon.HTTP_200
            return
        json_data = json.loads(body.decode())
        _NETWORKS[network_id] = vlan_id = json_data['vlan']

        bridge_name = L2Manager.get_bridge_name(network_id)
        L2Manager.add_bridge(bridge_name, vlan_id, False, False, resp)

        # XXX fixme
        resp.status = falcon.HTTP_200

    def on_delete(self, _, resp, network_id):

        if network_id in _NETWORKS:
            bridge_name = L2Manager.get_bridge_name(network_id)
            vlan_id = _NETWORKS[network_id]

            L2Manager.delete_bridge(bridge_name, vlan_id, False, 1, resp)

            del _NETWORKS[network_id]
        # XXX fixme
        resp.status = falcon.HTTP_200


class BridgeCmd(object):

    _NEWBRIDGE_REF_CNT = 0

    def on_put(self, req, resp, bridge_name, vlan_id):
        body = req.stream.read()
        if not body:
            spf = False
            newbridge = True
        else:
            json_data = json.loads(body.decode())
            spf = json_data.get('spf', False)
            newbridge = json_data.get('newbridge', True)

        if (bridge_name, vlan_id, 'newbridge') not in _BRIDGES:
            _BRIDGES[bridge_name, vlan_id, 'spf'] = spf
            _BRIDGES[bridge_name, vlan_id, 'newbridge'] = newbridge
            L2Manager.add_bridge(bridge_name, vlan_id, spf, newbridge, resp)

            if newbridge:
                self._NEWBRIDGE_REF_CNT = self._NEWBRIDGE_REF_CNT + 1

        # XXX fixme
        resp.status = falcon.HTTP_200

    def on_delete(self, _, resp, bridge_name, vlan_id):

        if (bridge_name, vlan_id, 'newbridge') in _BRIDGES:
            newbridge = _BRIDGES[bridge_name, vlan_id, 'newbridge']
            if newbridge:
                refcnt = self._NEWBRIDGE_REF_CNT
            else:
                refcnt = 1
            L2Manager.delete_bridge(bridge_name, vlan_id,
                                    newbridge, refcnt,
                                    resp)

            del _BRIDGES[bridge_name, vlan_id, 'newbridge']
            del _BRIDGES[bridge_name, vlan_id, 'spf']

            if newbridge and self._NEWBRIDGE_REF_CNT:
                self._NEWBRIDGE_REF_CNT = self._NEWBRIDGE_REF_CNT - 1

        # XXX fixme
        resp.status = falcon.HTTP_200


class VxlanCmd(object):

    def on_put(self, _, resp, network_id, vni):
        bridge_name = L2Manager.get_bridge_name(network_id)
        L2Manager.add_vxlan(vni, bridge_name, 0, False, resp)

        # XXX fixme
        resp.status = falcon.HTTP_200

    def on_delete(self, _, resp, network_id, vni):

        L2Manager.delete_vxlan(vni, resp)

        # XXX fixme
        resp.status = falcon.HTTP_200


class BridgeVxlanCmd(object):

    def on_put(self, _, resp, bridge_name, vlan_id, vni_id):

        if (bridge_name, vlan_id, 'newbridge') in _BRIDGES:
            newbridge = _BRIDGES[bridge_name, vlan_id, 'newbridge']

            L2Manager.add_vxlan(vni_id, bridge_name, vlan_id,
                                newbridge, resp)

        # XXX fixme
        resp.status = falcon.HTTP_200

    def on_delete(self, _, resp, bridge_name, vlan_id, vni_id):

        if (bridge_name, vlan_id, 'newbridge') in _BRIDGES:
            newbridge = _BRIDGES[bridge_name, vlan_id, 'newbridge']
            L2Manager.delete_vxlan(vni_id, newbridge, resp)

        # XXX fixme
        resp.status = falcon.HTTP_200


class IntfCmd(object):

    def on_put(self, _, resp, network_id, host):
        physical_interface = L2Manager.convert_host_to_interface(host)
        if (physical_interface is None) or (physical_interface == 'eth0'):
            # XXX fixme
            resp.status = falcon.HTTP_200
            return

        _PHYSICAL_INTERFACES[host] = physical_interface
        vlan_id = _NETWORKS[network_id]

        bridge_name = L2Manager.get_bridge_name(network_id)
        subinterface_name = L2Manager.add_vlan(physical_interface, vlan_id,
                                               resp)
        L2Manager.add_interface(bridge_name, subinterface_name, vlan_id,
                                False, False, resp)

        # XXX fixme
        resp.status = falcon.HTTP_200

    def on_delete(self, _, resp, network_id, host):

        physical_interface = _PHYSICAL_INTERFACES.get(host)
        if physical_interface is None:
            physical_interface = L2Manager.convert_host_to_interface(host)
            if (physical_interface is None) or (physical_interface == 'eth0'):
                # XXX fixme
                resp.status = falcon.HTTP_200
                return

        vlan_id = _NETWORKS[network_id]
        bridge_name = L2Manager.get_bridge_name(network_id)

        subinterface_name = L2Manager.delete_interface(bridge_name,
                                                       physical_interface,
                                                       vlan_id, False, True,
                                                       resp)

        L2Manager.delete_vlan(subinterface_name, resp)

        # XXX fixme
        resp.status = falcon.HTTP_200


class BridgeIntfCmd(object):

    __INTERFACE_INFO = {}

    def on_put(self, _, resp, bridge_name, vlan_id, host):
        physical_interface = L2Manager.convert_host_to_interface(host)

        if (physical_interface is None) or (physical_interface == 'eth0'):
            # XXX fixme
            resp.status = falcon.HTTP_200
            return

        if (bridge_name, vlan_id, 'newbridge') in _BRIDGES:
            newbridge = _BRIDGES[bridge_name, vlan_id, 'newbridge']

            if newbridge:
                interface_name = physical_interface
            else:
                interface_name = L2Manager.add_vlan(physical_interface,
                                                    vlan_id, resp)

            if (host, 'phyintf') in self.__INTERFACE_INFO:
                if vlan_id not in self.__INTERFACE_INFO[host, 'vlan']:
                    self.__INTERFACE_INFO[host, 'vlan'].append(vlan_id)
                    L2Manager.add_interface(bridge_name, interface_name,
                                            vlan_id, newbridge, False, resp)
            else:
                self.__INTERFACE_INFO[host, 'phyintf'] = physical_interface
                self.__INTERFACE_INFO[host, 'vlan'] = [vlan_id]
                L2Manager.add_interface(bridge_name, interface_name,
                                        vlan_id, newbridge, False, resp)

        # XXX fixme
        resp.status = falcon.HTTP_200

    def on_delete(self, _, resp, bridge_name, vlan_id, host):

        physical_interface = None
        if (host, 'phyintf') in self.__INTERFACE_INFO:
            physical_interface = self.__INTERFACE_INFO[host, 'phyintf']

        if physical_interface is None:
            physical_interface = L2Manager.convert_host_to_interface(host)
            if (physical_interface is None) or (physical_interface == 'eth0'):
                # XXX fixme
                resp.status = falcon.HTTP_200
                return

        if (bridge_name, vlan_id, 'newbridge') in _BRIDGES:
            newbridge = _BRIDGES[bridge_name, vlan_id, 'newbridge']

            if (host, 'phyintf') in self.__INTERFACE_INFO:
                if vlan_id in self.__INTERFACE_INFO[host, 'vlan']:
                    self.__INTERFACE_INFO[host, 'vlan'].remove(vlan_id)
                    if not self.__INTERFACE_INFO[host, 'vlan']:
                        del self.__INTERFACE_INFO[host, 'phyintf']
                        del self.__INTERFACE_INFO[host, 'vlan']
                        is_delete = True
                    else:
                        is_delete = False
            else:
                is_delete = True

            interface_name = L2Manager.delete_interface(bridge_name,
                                                        physical_interface,
                                                        vlan_id, newbridge,
                                                        is_delete, resp)

            if newbridge is None:
                L2Manager.delete_vlan(interface_name, resp)

        # XXX fixme
        resp.status = falcon.HTTP_200


class HashCmd(object):

    def on_get(self, _, resp):
        resp.status = falcon.HTTP_200
        resp.body = json.dumps(HASH_M.get_hash_id())
