# Copyright (C) 2015, 2016, 2017 Cumulus Networks, Inc. All rights reserved
#
# This software is subject to the Cumulus Networks End User License Agreement available
# at the following locations:
#
# Internet: https://cumulusnetworks.com/downloads/eula/latest/view/
# Cumulus Linux systems: /usr/share/cumulus/EULA.txt

from restapi import settings
from restapi.utils.logs import get_logger
import falcon
import json
import mimetypes
import os


_LOG = get_logger()


class APIInfo(object):
    """Show the main information about the API like endpoints, version, etc.
    """
    def on_get(self, req, res):
        from restapi.urls import expand
        res.status = falcon.HTTP_200
        res.body = json.dumps(
            {
                "endpoints": dict(
                    (name, uri) for uri, _, name in expand()
                ),
                "version": {
                    "api_version": settings.__version__,
                    "api_codename": settings.__codename__,
                    "api_status": settings.__status__,
                    "documentation": settings.__docs__
                }
            }
        )

    def on_options(self, _, res):
        """Acknowledge the OPTIONS method.
        """
        res.status = falcon.HTTP_200

    def on_post(self, req, res):
        raise falcon.HTTPError(falcon.HTTP_405,
                               title="Client error",
                               description="{0} method not allowed.".format(
                                   req.method),
                               href=settings.__docs__)

    def on_put(self, req, res):
        raise falcon.HTTPError(falcon.HTTP_405,
                               title="Client error",
                               description="{0} method not allowed.".format(
                                   req.method),
                               href=settings.__docs__)

    def on_update(self, req, res):
        raise falcon.HTTPError(falcon.HTTP_405,
                               title="Client error",
                               description="{0} method not allowed.".format(
                                   req.method),
                               href=settings.__docs__)

    def on_delete(self, req, res):
        raise falcon.HTTPError(falcon.HTTP_405,
                               title="Client error",
                               description="{0} method not allowed.".format(
                                   req.method),
                               href=settings.__docs__)


class StaticSink(object):

    def __call__(self, req, res):
        if not req.relative_uri.startswith(settings.STATIC_PATH):
            return
        path = req.relative_uri[len(settings.STATIC_PATH):].strip('/')
        if not path:
            resource = os.path.join(settings.STATIC_FILES, 'index.html')
        else:
            resource = os.path.join(settings.STATIC_FILES, path)
            if not os.path.exists(resource):
                _LOG.warn('Static file %s not found', resource)
                raise falcon.HTTPNotFound()
        filetype = mimetypes.guess_type(resource, strict=True)[0]
        if not filetype:
            filetype = 'text/plain'
        res.content_type = filetype
        # os.environ['wsgi.file_wrapper'](open(resource, 'rb'), 4096)
        res.stream = open(resource, 'rb')
        return res

