# Copyright (C) 2015, 2016, 2017 Cumulus Networks, Inc. All rights reserved
#
# This software is subject to the Cumulus Networks End User License Agreement available
# at the following locations:
#
# Internet: https://cumulusnetworks.com/downloads/eula/latest/view/
# Cumulus Linux systems: /usr/share/cumulus/EULA.txt

from os.path import isfile
from oslo.config import cfg
import sys


class ConfigSection(object):

    AGENT = 'AGENT'
    NETQ = 'NETQ'
    LOGGING = 'LOGGING'
    SERVER = 'SERVER'
    ML2 = 'ML2'

    def __init__(self):
        raise NotImplementedError

_AGENT_OPTS = [
    cfg.StrOpt('root_helper',
               default='sudo',
               help='Root helper application.'),
]

_SERVER_OPTS = [
    cfg.BoolOpt('daemon',
                short='D',
                default=False,
                help='Start in the foreground instead of daemonizing.'),
    cfg.StrOpt('bind',
               short='b',
               default='unix:/var/run/httpapi/uds',
               help='The Unix domain socket on which '
                    'the server will listen for HTTP requests.'),
    cfg.StrOpt('server-config',
               default='/etc/gunicorn.conf.py',
               help='The path to a Gunicorn config file, or python module'),
    cfg.StrOpt('pidfile',
               short='p',
               default='/var/run/restserver.pid',
               help='The location in which the server keeps its pid file. '
                    'This directory must be writable.'),
    cfg.StrOpt('keep-alive',
               default='300',
               help='The number of seconds to wait for requests on a Keep-Alive connection.'),
    # Warning: This value must be "root" or "www-data" or Gunicorn will not be able to read and
    # write from/to the socket.  If the user is not "root", executing commands remotely as
    # "root" will not be possible.
    cfg.StrOpt('user',
               short='u',
               default='root',
               help='The user for Gunicorn worker processes.'),
]

_LOGGING_OPTS = [
    cfg.StrOpt('logpath',
               default='syslog',
               help='Where to store log files. This directory must be '
                    'writable by the agent.'),
    cfg.StrOpt('loglevel',
               default='INFO',
               choices=['DEBUG', 'INFO', 'WARNING', 'ERROR', 'CRITICAL'],
               help='Log level'),
]

_NETQ_OPTS = [
    cfg.StrOpt('server_address',
               default='7.0.0.1',
               help='Redis server address.'),
]

_ML2_OPTS = [
    cfg.ListOpt('trunk_interfaces', default=None,
                help=('List of trunk interfaces')),
    cfg.StrOpt('local_bind', default='No data',
               help=('VxLAN bind port')),
    cfg.StrOpt('service_node', default='No data',
               help=('VxLan service node')),
    cfg.BoolOpt('use_net', default='False',
               help=('Use net command for configuration')),
]


def get_pidfile():
    print((cfg.CONF.pidfile))


def get_root_helper():
    return cfg.CONF.AGENT.root_helper


def setup_conf():
    __NETBIN = '/usr/bin/net'

    cfg.CONF.register_cli_opts(_SERVER_OPTS)
    cfg.CONF.register_opts(_LOGGING_OPTS, ConfigSection.LOGGING)
    cfg.CONF.register_opts(_AGENT_OPTS, ConfigSection.AGENT)
    cfg.CONF.register_opts(_NETQ_OPTS, ConfigSection.NETQ)
    cfg.CONF.register_opts(_ML2_OPTS, ConfigSection.ML2)
    try:
        cfg.CONF(default_config_files=['/etc/restapi.conf'],
                 validate_default_values=True)
    except cfg.ConfigFilesNotFoundError:
        sys.stderr.write('Failed to load configuration file')

    if cfg.CONF.ML2.use_net and not isfile(__NETBIN):
        cfg.CONF.ML2.use_net = False
