# Copyright (C) 2015, 2016, 2017 Cumulus Networks, Inc. All rights reserved
#
# This software is subject to the Cumulus Networks End User License Agreement available
# at the following locations:
#
# Internet: https://cumulusnetworks.com/downloads/eula/latest/view/
# Cumulus Linux systems: /usr/share/cumulus/EULA.txt

from restapi.utils import config
from restapi.utils.logs import get_logger
import falcon
import json
import six
import subprocess


if six.PY3:
    import asyncio

_LOG = get_logger()
_shell = None


class _Shell27(object):

    def __init__(self, root_helper):
        self.root_helper = root_helper

    def __call__(self, args):
        if isinstance(args, str):
            args = args.split()
        process = subprocess.Popen([self.root_helper] + args,
                                   stdout=subprocess.PIPE,
                                   stderr=subprocess.STDOUT)
        stdout, _ = process.communicate()
        return stdout, process.returncode


class _Shell(object):

    def __init__(self, root_helper):
        self.root_helper = root_helper

    def __call__(self, args):
        if six.PY3():
            loop = asyncio.get_event_loop()
            return loop.run_until_complete(self.exec_command(args))
        else:
            if isinstance(args, str):
                args = args.split()
            process = subprocess.Popen([self.root_helper] + args,
                                       stdout=subprocess.PIPE,
                                       stderr=subprocess.STDOUT)
            stdout, _ = process.communicate()
            return stdout, process.returncode

    def exec_command(self, args):
        pass

if six.PY3:
    _Shell.exec_command = asyncio.coroutine(_Shell.exec_command)


def execute_command(cmd, resp):
    """ error: stderr if unsuccessful
               type: string
               optional: yes
        output: stdout of the command if successful
                type: string
                optional: yes
    """
    global _shell
    _shell = _shell or _Shell27(config.get_root_helper())
    _LOG.debug('Executing %s', cmd)
    output, retcode = _shell(cmd)
    _LOG.debug('Return code %s, output %s', retcode, output)
    # cmd_response = CommandResponse()
    # cmd_response.code = retcode
    # cmd_response.message = output
    resp.status = falcon.HTTP_200
    # cmd_response.type = 'stdout'
    if retcode != 0:
        resp.status = falcon.HTTP_500
        _LOG.error('Command failed. out:%s', output)
        # cmd_response.type = 'stderr'
    try:
        resp.body = json.dumps(output)
    except Exception:
        resp.body = output

def execute_command_no_resp(cmd):
    """ error: stderr if unsuccessful
               type: string
               optional: yes
        output: stdout of the command if successful
                type: string
                optional: yes
    """
    global _shell
    _shell = _shell or _Shell27(config.get_root_helper())
    _LOG.debug('Executing %s', cmd)
    output, retcode = _shell(cmd)
    _LOG.debug('Return code %s, output %s', retcode, output)

    if retcode != 0:
        return (output, False)
    else:
        return (output, True)
